/***************************************************************
From the book "Motif Programming, The Essentials...and More"
by Marshall Brain.
Published by Digital Press, ISBN 1-55558-089-0.
To order the book, call 1-800-DIGITAL and ask for EY-J816E-DP.

Copyright 1992, by Digital Equipment Corp.

This code demonstrates the creation of a popup menu on the
third mouse button.
***************************************************************/

/*popup.c*/

#include <Xm/Xm.h>
#include <Xm/DrawingA.h>
#include <Xm/PushB.h>
#include <Xm/RowColumn.h>

#define LINE 1
#define BOX  2

XtAppContext context;
XmStringCharSet char_set=XmSTRING_DEFAULT_CHARSET;

Widget toplevel;
Widget drawing_area;
Widget menu;
Widget line_item;
Widget box_item;

GC gc_copy;
GC gc_xor;
int start_x,start_y;/* starting position of shape */
int old_x,old_y;/* previous cursor position   */
int current_shape=LINE;
Boolean shape_started=False;

void setup_gcs()
/* set up the graphics context. */
{
    int foreground,background;
    Arg al[10];
    int ac;
    XGCValues vals;

    /* get the current fg and bg colors. */
    ac=0;
    XtSetArg(al[ac], XmNforeground, &foreground); ac++;
    XtSetArg(al[ac], XmNbackground, &background); ac++;
    XtGetValues(drawing_area, al, ac);

    /* create the copy gc. */
    vals.foreground = foreground;
    vals.background = background;
    gc_copy= XtGetGC(drawing_area, GCForeground | GCBackground, &vals);

    /* create the xor gc. */
    vals.foreground = foreground ^ background;
    vals.function = GXxor;
    gc_xor= XtGetGC(drawing_area, GCForeground | GCBackground |
        GCFunction, &vals);
}

void draw_shape(w,gc)
    Widget w;
    GC gc;
{
    switch (current_shape)
    {
        case LINE:
            XDrawLine(XtDisplay(w), XtWindow(w), gc, 
                start_x, start_y, old_x, old_y);
            break;
        case BOX:
            XDrawRectangle(XtDisplay(w), XtWindow(w), gc, 
                start_x, start_y, old_x-start_x, old_y-start_y);
            break;
    }
}

void handle_start(w,client_data,event)
    Widget w;
    XtPointer client_data;
    XEvent *event;
/* handles the ButtonPress event */
{
    shape_started=False;
    if (event->xbutton.button==Button1)
    {
        old_x=start_x=event->xbutton.x;
        old_y=start_y=event->xbutton.y;
        draw_shape(w,gc_xor);
shape_started=True;
    }
    else if (event->xbutton.button==Button3)
    {
        XmMenuPosition(menu,(XButtonPressedEvent *)event);
        XtManageChild(menu);
    }
}

void handle_drag(w,client_data,event)
    Widget w;
    XtPointer client_data;
    XEvent *event;
/* handles the ButtonMotion event */
{
    if (shape_started)
    {
        draw_shape(w,gc_xor);
        old_x=event->xbutton.x;
        old_y=event->xbutton.y;
        draw_shape(w,gc_xor);
    }
}

void handle_done(w,client_data,event)
    Widget w;
    XtPointer client_data;
    XEvent *event;
/* handles the ButtonRelease event */
{
    if (event->xbutton.button==Button1)
    {
        old_x=event->xbutton.x;
        old_y=event->xbutton.y;
        draw_shape(w,gc_copy);
        shape_started=False;
    }
}

void menuCB(w,client_data,call_data)
    Widget w;
    int client_data;
    XmAnyCallbackStruct *call_data;
/* callback routine used for all menus */
{
    current_shape=client_data;
    XtUnmanageChild(menu);
}

Widget make_menu_item(item_name,client_data,menu)
    char *item_name;
    XtPointer client_data;
    Widget menu;
/* adds an item into a menu. */ 
{
    int ac;
    Arg al[10];
    Widget item;

    ac = 0;
    XtSetArg(al[ac],XmNlabelString,
        XmStringCreateLtoR(item_name,char_set)); ac++;  
    item=XmCreatePushButton(menu,item_name,al,ac);
    XtManageChild(item);
    XtAddCallback(item,XmNactivateCallback,menuCB,client_data);  
    XtSetSensitive(item,True);
    return(item);
}

void main(argc,argv)
    int argc;
    char *argv[];
{
    Arg al[10];
    int ac;

    /* create the toplevel shell */
    toplevel = XtAppInitialize(&context,"",NULL,0,&argc,argv,
        NULL,NULL,0);

    /* default window size. */
    ac=0;
    XtSetArg(al[ac],XmNheight,400); ac++;
    XtSetArg(al[ac],XmNwidth,400); ac++;
    XtSetValues(toplevel,al,ac);

    /* create drawing area */
    ac=0;
    drawing_area=XmCreateDrawingArea(toplevel,"drawing_area",al,ac);
    XtManageChild(drawing_area);
    /* add in event handlers for drawing area */
    XtAddEventHandler(drawing_area, ButtonPressMask, FALSE,
        handle_start, NULL);
    XtAddEventHandler(drawing_area, ButtonReleaseMask, FALSE,
        handle_done, NULL);
    XtAddEventHandler(drawing_area, ButtonMotionMask, FALSE,
        handle_drag, NULL);

    /* create the popup menu */
    ac=0;
    menu=XmCreatePopupMenu(drawing_area,"menu",al,ac);
    line_item=make_menu_item("Line",LINE,menu);
    box_item=make_menu_item("Box",BOX,menu);

    setup_gcs();

    XtRealizeWidget(toplevel);
    XtAppMainLoop(context);
}
